##*******************************************
## Pepper.coffee
## Peppermint Coffee API
##
## Coded by Dr.Kameleon
##
## Copyright (c) 2011-2014 InSili.co
## All rights reserved.
##*******************************************

##===================================
## Private Classes
##===================================

class _Core

    constructor: ()->
        @currentPlugin = null;
        @currentDocument = new _Document;
        @version = "1.4 (Build 2652)";
        
    coffee: (code,bare=false,exec=false)->
        try 
            if typeof code == 'object'
                code = eval(code)[0]
            result = CoffeeScript.compile(code, { bare: bare });
            if exec
                eval.apply(window,[result]);
            else 
                return result;
        catch error
            $Terminal.echo("Coffee:: Error : #{ error.toString() }");

    less: (code,name)->
        try
            if typeof code == 'object'
                code = eval(code)[0]

            if name?
                code = code.replace(new RegExp("(\.[a-z]+)","gm"), ".ace_$1");
                code = code.replace(new RegExp("(##{ name })","gm"), ".ace-#{ name }");
                
            parser = window.less.Parser(); 
            parser.parse(code, (error,result)=>
                if !error
                    return result.toCSS();
            );

        catch error
            $Terminal.echo("LESS:: Error : #{ error.toString() }");

        return "";
        
            
    registerPluginMenu: ()->
        core.registerPluginMenu_(JSON.stringify(Object.keys(@currentPlugin.menu())));
        
    loadPlugin: (name,path,source)->
        if typeof code == 'object'
            source = eval(code)[0]
        @coffee("#{ source }\n\nwindow.#{ name } = #{ name } unless window.#{ name };",false,true);
        
        @currentPlugin = new window[name](name,path);
        @registerPluginMenu();
        @currentPlugin.init();
        
    haltPlugin: ()->
        if @currentPlugin?
            @currentPlugin.halt();
            $Events.removeById(@currentPlugin.name);
            window[@currentPlugin.name] = null;
            @currentPlugin = null;
        
            
    runPlugin: (name,path,source)->
        if @currentPlugin? and @currentPlugin.name != name
            @haltPlugin();
    
        if !@currentPlugin? or @currentPlugin.name != name
            @loadPlugin(name,path,source);
            
        @currentPlugin.run();
        
    testPlugin: (refresh=false)->
        pluginName = $Document.info().filename.replace($Document.info().extension,"").replace(".","");
        pluginPath = $Document.info().folder;
        pluginContent = $Document.info().content;
    
        if refresh
            @haltPlugin();
        
        @runPlugin(pluginName, pluginPath, pluginContent);

class _Events

    constructor: ()->
        @list = {
            documentChanged : {},
            documentSaved   : {},
            cursorChanged   : {}
            callback        : {}
        }
        
    exec: (event,id,args)->
        for eventId,callback of @list[event]
            if !id?
                callback(args);
            else
                if id==eventId
                    callback(args);
        
    add: (event,id,callback)->
        if !id?
            id = pp.uuid();
            
        @list[event][id] = callback;
        return "$Events.exec('callback','#{ id }','%@')";
        
    remove: (event, id)->
        delete @list[event][id];
        
    removeById: (id)->
        for event,content of @list
            for eventId,callback of @list[event]
                if id==eventId
                    @remove(event,id)
        
    set: (event,id,callback)->
        if callback?
            @add(event,id,callback);
        else
            @remove(event,id,callback);
            
##===================================
## Public Classes
##===================================

###
@class Dialog
@description The Dialog object - used to invoke dialogs
###

class _Dialog

    constructor: ()->
        
    ###
    @method alert
    @description Display an alert dialog
    @param title|required|string|The main dialog title
    @param info|optional|string|Related information
    @param callback|optional|function|The function to be called, when the dialog closes
    @example $Dialog.alert("Finished!","The operation has finished")
    ###

    alert: (title,info="",callback)->
        if callback?
            event = $Events.set("callback", null, callback);
        else
            event=null;
        core.dialogAlert_info_callback_(title,info,event);
        
    ###
    @method input
    @description Show an input dialog and get user input
    @param title|required|string|The title/prompt of the dialog
    @param callback|required|function|The function to be called, with the input, when the dialog closes
    @return null|
    @example $Dialog.input("Enter a number:", (number)=>
    @example    $Terminal.echo("Got : #{ number }");    
    @example );
    ###
    
    input: (title,callback)->
        event = $Events.set("callback", null, callback);
        core.dialogInput_callback_(title,event);
        
###
@class Document
@description The current Document object
@property label|string|The document's label as seen in Peppermint
@property path|string|The full path of the document
@property filename|string|The document's pure filename
@property folder|string|The document's pure folder name
@property extension|string|The document's file extension
###
        
class _Document

    constructor: ()->
        @content = null;
        
    ###
    @method info
    @description Get/Update Info of current document and/or create a temporary copy for processing
    @param ext|optional|string|Force a particular extension (if document is to be created)
    @return Document|A copy of the updated Document object, with all its properties populated
    @example path = $Document.info().path
    ###
        
    info: (ext="tmp")->
    
        if !@content? or pp.getContent()!=@content or core.getLabel()!=@label
            @content = pp.getContent();
            @label = core.getLabel();
            @path = pp.getFullPath();
            if @path==""
                @path = core.getTempFile_withExtension_(@content, ext);
            @filename = @path.replace(/^.*[\\\/]/, '');
            @folder = @path.replace(@filename, '');
            @extension = /(?:\.([^.]+))?$/.exec(@filename)[1];
        return @

    ###
    @method temp
    @description Create temporary document instance with contents
    @param content|required|string|The document's content
    @return Document|A Document object, with all its properties populated
    @example temp = $Document.temp("my contents");
    @example path = temp.path;
    ###
    temp: (content)->
        @path = core.getTempFile_withExtension_(content, "tmp");
        @content = content;
        @filename = @path.replace(/^.*[\\\/]/, '');
        @label = @filename;
        @folder = @path.replace(@filename, '');
        @extension = /(?:\.([^.]+))?$/.exec(@filename)[1];
        return @
            
        
###
@class Editor
@description The main Event-handling object
###

class _Editor

    constructor: ()->
        
    ###
    @method bind
    @description Create or Destroy and editor Key Binding
    @param key|required|string|Key combination
    @param callback|optional|function|Callback on key combination (null to destroy)
    @return null|
    @example $Editor.bind("Cmd+R", ()=>
    @example    $Terminal.echo("Running");
    @example );
    ###
        
    bind: (key,callback=null)->
        pp.editor.commands.bindKey(key,callback);
        
    ###
    @method content
    @description Get/Set editor content
    @param value|optional|string|New editor content
    @param undoable|optional|boolean|Set if change will be undoable (default: true)
    @return null or string|The content of the editor
    @example currentContent = $Editor.content();
    @example 
    @example # Let's change it 
    @example $Editor.content("New content");
    ###
        
    content: (value,undoable=true)->
        if value?
            if undoable
                pp.setUndoableContent(value);
            else
                pp.setContent(value);
        else
            return pp.getContent();
            
    ###
    @method selection
    @description Get/Set current editor selection
    @param value|optional|string|Replace current selection with value
    @return null or string|The currently selected text
    @example currentContent = $Editor.content();
    @example 
    @example # Let's change it 
    @example $Editor.content("New content");
    ###
            
    selection: (value)->
        if value?
            pp.setSelection(value);
        else
            return pp.getSelection();
            
    option: {
        
        ###
        @method option.gutter
        @description Get/Set Gutter state
        @param onoff|optional|boolean|True/False
        @return null or boolean|
        ###
            
        gutter: (onoff)->
            if onoff?
                pp.toggleGutter(onoff);
            else
                return pp.editor.renderer.getShowGutter();
                
        ###
        @method option.printMargin
        @description Get/Set Print Margin state
        @param onoff|optional|boolean|True/False
        @return null or boolean|
        ###
                
        printMargin: (onoff)->
            if onoff?
                pp.togglePrintMargin(onoff);
            else
                return pp.editor.getShowPrintMargin();
                
        ###
        @method option.indentGuides
        @description Get/Set Indentation Guides state
        @param onoff|optional|boolean|True/False
        @return null or boolean|
        ###
                
        indentGuides: (onoff)->
            if onoff?
                pp.toggleIndentationGuides(onoff);
            else
                return pp.editor.renderer.getDisplayIndentGuides();
                
        ###
        @method option.invisibles
        @description Get/Set Invisibles state
        @param onoff|optional|boolean|True/False
        @return null or boolean|
        ###
                
        invisibles: (onoff)->
            if onoff?
                pp.toggleInvisibles(onoff);
            else
                return pp.editor.getShowInvisibles();
                
        ###
        @method option.softTabs
        @description Get/Set Soft Tabs state
        @param onoff|optional|boolean|True/False
        @return null or boolean|
        ###
                
        softTabs: (onoff)->
            if onoff?
                pp.toggleSoftTabs(onoff);
            else
                return pp.editor.getUseSoftTabs();
                
        ###
        @method option.wordWrap
        @description Get/Set Word Wrap state
        @param onoff|optional|boolean|True/False
        @return null or boolean|
        ###
                
        wordWrap: (onoff)->
            if onoff?
                pp.toggleWordWrap(onoff);
            else
                return pp.editor.getSession().getUseWrapMode();
                
        ###
        @method option.tabSize
        @description Get/Set Tab Size
        @param size|optional|number|Tab size in characters
        @return null or number|
        ###
                
        tabSize: (size)->
            if size?
                pp.setTabSize(size);
            else
                return pp.editor.getSession().getTabSize();
                
        ###
        @method option.wrapLimit
        @description Get/Set Wrap Limit
        @param size|optional|number|Wrap Limit in characters
        @return null or number|
        ###
                
        wrapLimit: (limit)->
            if limit?
                pp.setWrapLimit(limit);
            else
                return pp.editor.getSession().getWrapLimitRange().min;
                
    }
            
###
@class On
@description The main Event-handling object
###
        
class _On

    constructor: (pluginName)->
        @pluginId = pluginName;
        
    ###
    @method documentChanged
    @description Set callback to be executed, when document changes
    @param callback|required|function|The callback to be executed, or null to remove handler
    @return null|
    @example init: ()->
    @example    $$x = 1;
    @example    $$on.documentChanged(()=>
    @example        $Terminal.echo("Document changed");
    @example        $Terminal.echo("x = " + $$x++);
    @example    );
    @example  
    @example halt: ()->
    @example    $$on.documentChanged(null);
    ###
    
    documentChanged: (callback)->
        $Events.set("documentChanged", @pluginId, callback);
        null
        
    ###
    @method documentSaved
    @description Set callback to be executed, when document is saved
    @param callback|required|function|The callback to be executed, or null to remove handler
    @return null|
    @example $$on.documentSaved(()=>
    @example    # do something
    @example )
    ###
        
    documentSaved: (callback)->
        $Events.set("documentSaved", @pluginId, callback);
        null
        
    ###
    @method cursorChanged
    @description Set callback to be executed, when document changes
    @param callback|required|function|The callback to be executed, or null to remove handler
    @return null|
    @example $$on.cursorChanged(()=>
    @example    # do something
    @example )
    ###

    cursorChanged: (callback)->
        $Events.set("cursorChanged", @pluginId, callback);
        null
        
###
@class Plugin
@description The main Inheritable Plugin class
@property name|string|The plugin's name as defined in the <code>info.json</code>
@property path|string|The plugin's folder path
@property on|On|An associated <a href='#api-class-on'>On</a> Object for event handling
###
        
class _Plugin
    
    constructor: (name,path)->
        @name = name;
        @path = path;
        @bin  = window.binariesIndex;
        
        @on = new _On(@name);
        
    ###
    @method init
    @description Called automatically when plugin is initialized the first time
    @return null|
    @example class MyPlugin extends $Plugin
    @example    init:->
    @example        $Terminal.echo("Plugin initialised!");
    ###
    
    init: ()->
        
    ###
    @method run
    @description Called every time the plugin is executed
    @return null|
    @example class MyPlugin extends $Plugin
    @example    init:->
    @example        $Terminal.echo("Plugin initialised!");
    @example    
    @example    run:->
    @example        $Terminal.echo("Plugin is running!");
    ###
        
    run: ()->
        
    ###
    @method menu
    @description Called automatically when plugin is initialized the first time
    @return object|A dictionary of menu items associated with the plugin (Menu Item =&gt; callback)
    @example class MyPlugin extends $Plugin
    @example    init:->
    @example        $Terminal.echo("Plugin initialised!");
    @example    
    @example    doSomething:(arg)->
    @example        $Terminal.echo("Doing something with #{ arg }");
    @example
    @example    menu:-> {
    @example        "First menu item"   :=> @doSomething("first")
    @example        "Second menu item"  :=> @doSomething("second")
    @example        "Third menu item"   :=> @doSomething("third")
    @example    }
    ###
        
    menu: ()-> {}
    
    ###
    @method halt
    @description Called automatically when plugin is terminated, or another plugin is executed
    @return null|
    @example class MyPlugin extends $Plugin
    @example    init:->
    @example        $Terminal.echo("Plugin initialised!");
    @example    
    @example    halt:->
    @example        $Terminal.echo("Plugin is being terminated...");
    ###
        
    halt: ()->
    
    ###
    @method import
    @description Import a local file's contents
    @param path|required|string|The path to the file
    @param relative|option|boolean|Is the path relative to the plugin folder? (default: true)
    @return string|The file contents
    ###
        
    import: (path,relative=true)-> 
        if relative
            path = @path + "/" + path;
            
        return core.file_(path);
        
    ###
    @method include
    @description Include a local script file (JavaScript or CoffeeScript) and execute it in global scope
    @param path|required|string|The path to the file
    @param relative|option|boolean|Is the path relative to the plugin folder? (default: true)
    @return string|The file contents
    ###
            
    include: (path,relative=true)-> 
        if relative
            path = @path + "/" + path;

        core.importjs_(path);

        
###
@class Terminal
@description The main Terminal object - representing the console panel
@property verbose|boolean|Display more info <i>(Default: false)</i>
###
        
class _Terminal

    constructor: ()->
        @environment = {};
        @verbose = false;
        
    ###
    @method show
    @description Show the terminal panel, if not already open
    @return null|
    @example $Terminal.show()
    ###
        
    show: ()-> 
        pp.showConsoleTab();
        null
        
    ###
    @method clear
    @description Clear the terminal panel
    @return null|
    @example $Terminal.clear()
    ###
        
    clear: ()-> 
        terminal.clear();
        null
        
    ###
    @method echo
    @description Echoes given string to the terminal
    @param message|required|string|The message to be displayed
    @param attributes|optional|string|A style attribute string
    @return null|
    @example $Terminal.echo("Hello world!")
    ###    
    
    echo: (msg,attributes)->
        if attributes?
            msg = "[[#{ attributes }]#{ msg }]";
            
        if typeof msg == 'object'
                msg = eval(msg)[0]
            
        terminal.echo(msg);
        null
        
    ###
    @method exec
    @description Asynchronously execute task and print output in terminal
    @param args|required|string...|Command/Arguments to be passed
    @param completed|optional|callback|Callback to be executed when task is completed
    @return string|The execution result
    @example $Terminal.exec("php","--version")
    ###
    
    exec: (args...)->
        if @verbose
            outp = "";
            for arg in args
                if typeof arg != 'function'
                    outp += arg + " "
            $Terminal.echo("# #{ outp }");
            
        if typeof args[args.length-1] == 'function'
            completed = args[args.length-1]
            args.pop();
        else
            completed = null

        args.push((output)=>
            if output.trim()!=""
                $Terminal.echo(output);
        );
        
        if completed?
            args.push(completed);
            
        @execSilent.apply(@,args);
        
    ###
    @method execSilent
    @description Execute command in terminal (either synchronously, or async)
    @param args|required|string...|Command/Arguments to be passed
    @param action|optional|callback|Callback to be executed when any data is served
    @param completed|optional|callback|Callback to be executed when task is completed
    @return string|The execution result
    @example $Terminal.execSilent("php","--version")
    @example
    @example # This is pretty much like `$Terminal.exec()`
    @example $Terminal.execSilent("php","myscript.php", (output)=>
    @example    $Terminal.echo("Got output: #{ output }");
    @example ,(result)=>
    @example    $Terminal.echo("Task terminated!");
    @example ); 
    ###
        
    execSilent: (args...)->

        action = null;
        completed = null;
        
        if typeof args[args.length-2] == 'function'
            action = $Events.set("callback", null, args[args.length-2]);
            completed = $Events.set("callback", null, args[args.length-1]);
            args.pop();
            args.pop();
        else
            if typeof args[args.length-1] == 'function'
                action = $Events.set("callback", null, args[args.length-1]);
                args.pop();
            
        args = JSON.stringify( args );
        
        if action!=null or completed!=null
            window.setTerminalStdin(true);
            
        if Object.keys(@environment).length
            return core.terminalExec_env_action_completed_(args, JSON.stringify(@environment), action, completed);
        else
            return core.terminalExec_env_action_completed_(args, "", action, completed);
        
            
    ###
    @method setVar
    @description Set custom environment variable
    @param variable|required|string|The name of the variable
    @param value|required|string|The value of the variable
    @return null|
    @example $Terminal.setVar("MYVAR","VALUE");
    ###
            
    setVar: (variable,value)->
        @environment[variable] = value;
        
    write: (msg)->
        core.terminalWrite_(msg);
        
    ###
    @method kill
    @description Kill existing async process being executed
    @return null|
    @example $Terminal.kill();
    ###
        
    kill: ()->
        core.terminalKill();
        
###
@class Utility
@description The main Utility object - representing the Utility Panel on the right side of the Peppermint window
###
        
class _Utility

    constructor: ()->
        
    ###
    @method load
    @description Load html tool in the Utility View
    @param title|required|string|The new title for the Utility View
    @param source|required|string|Path to the .html file to be loaded or html source
    @param base|optional|string|Base path to be used
    @return null|
    @example $Utility.load("My great tool", "#{ @path }/index.html")
    ###
        
    load: (title,source,base=null)->
        if !base?
            core.loadUtility_label_(source,title);
        else
            core.loadHtml_label_withBasePath_(source,title,base);
        
    ###
    @method loadDocument
    @description Load custom document in the Utility View
    @param title|required|string|The new title for the Utility View
    @param content|required|string|The script's contents
    @param spec|required|string|The spec to be used for highlighting
    @return null|
    @example $Utility.loadDocument("My PHP Script", "<?php echo 'done'; ?>", "php")
    ###
        
    loadDocument: (title,content,spec)->
        core.loadEditorUtility_label_mode_(content,title,spec);
        
        
##===================================
## Initialization
##===================================        

# Private

@$Core      = new _Core unless @$Core;
@$Events    = new _Events unless @$Events;

# Public 
        
@$Dialog    = new _Dialog unless @$Dialog;
@$Document  = new _Document unless @$Document;
@$Editor    = new _Editor unless @$Editor;
@$Terminal  = new _Terminal unless @$Terminal;
@$Utility   = new _Utility unless @$Utility;

# Inheritable

@$Plugin    = _Plugin unless @$Plugin;

##*******************************************
## This is the end,
## my only friend, the end...
##*******************************************
    
    